/* $Id: cpl_propertylist-test.c,v 1.32 2007/12/11 15:07:59 scastro Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2005 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: scastro $
 * $Date: 2007/12/11 15:07:59 $
 * $Revision: 1.32 $
 * $Name:  $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#undef CX_DISABLE_ASSERT
#undef CX_LOG_DOMAIN

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <math.h>

#include <fitsio.h>

#include <cxmemory.h>
#include <cxmessages.h>

#include "cpl_init.h"
#include "cpl_image_io.h"
#include "cpl_error.h"
#include "cpl_propertylist_impl.h"
#include "cpl_table.h"

#define FLT_EPS  1.0e-6
#define DBL_EPS  1.0e-14



/*static int cpl_test_property_comparison(
        const void  *   p1,
        const void  *   p2)*/
static int cpl_test_property_comparison(
        const cpl_property  *   p1,
        const cpl_property  *   p2)
{

    cpl_property *pro1 = (cpl_property *)p1;
    cpl_property *pro2 = (cpl_property *)p2;

    /*
    * Compare the two properties
    */
    return strcmp(cpl_property_get_name(pro1),
                  cpl_property_get_name(pro2));
}

static void
cpl_test_property_dump(cpl_property *property)
{

    const cxchar *name = cpl_property_get_name(property);
    const cxchar *comment = cpl_property_get_comment(property);

    cxchar c;

    long size = cpl_property_get_size(property);

    cpl_type type = cpl_property_get_type(property);


    fprintf(stderr, "Property at address %p\n", property);
    fprintf(stderr, "\tname   : %p '%s'\n", name, name);
    fprintf(stderr, "\tcomment: %p '%s'\n", comment, comment);
    fprintf(stderr, "\ttype   : %#09x\n", type);
    fprintf(stderr, "\tsize   : %ld\n", size);
    fprintf(stderr, "\tvalue  : ");


    switch (type) {
    case CPL_TYPE_CHAR:
        c = cpl_property_get_char(property);
        if (!c)
            fprintf(stderr, "''");
        else
            fprintf(stderr, "'%c'", c);
        break;

    case CPL_TYPE_BOOL:
        fprintf(stderr, "%d", cpl_property_get_bool(property));
        break;

    case CPL_TYPE_INT:
        fprintf(stderr, "%d", cpl_property_get_int(property));
        break;

    case CPL_TYPE_LONG:
        fprintf(stderr, "%ld", cpl_property_get_long(property));
        break;

    case CPL_TYPE_FLOAT:
        fprintf(stderr, "%.7g", cpl_property_get_float(property));
        break;

    case CPL_TYPE_DOUBLE:
        fprintf(stderr, "%.15g", cpl_property_get_double(property));
        break;

    case CPL_TYPE_STRING:
        fprintf(stderr, "'%s'", cpl_property_get_string(property));
        break;

    default:
        fprintf(stderr, "unknown.");
        break;

    }

    fprintf(stderr, "\n");

    return;

}


static void
cpl_test_plist_dump(cpl_propertylist *plist)
{

    cxlong i;
    cxlong sz = cpl_propertylist_get_size(plist);


    fprintf(stderr, "Property list at address %p:\n", plist);

    for (i = 0; i < sz; i++) {
        cpl_property *p = cpl_propertylist_get(plist, i);
        cpl_test_property_dump(p);
    }

    return;

}

int main(void)
{

    const cxchar *keys[] = {
        "a", "b", "c", "d", "e", "f", "g",
        "A", "B", "C", "D", "E", "F", "G"
    };

    const cxchar *comments[] = {
        "A character value",
        "A boolean value",
        "A integer value",
        "A long integer value",
        "A floating point number",
        "A double precision number",
        "A string value",
    };

    cpl_type types[] = {
        CPL_TYPE_CHAR,
        CPL_TYPE_BOOL,
        CPL_TYPE_INT,
        CPL_TYPE_LONG,
        CPL_TYPE_FLOAT,
        CPL_TYPE_DOUBLE,
        CPL_TYPE_STRING
    };

    cxint status;

    cxlong i;
    cxlong naxes[2] = {256,256};

    cxfloat fval[] = {-1.23456789, 0.};
    cxdouble dval[] = {-1.23456789, 0.};

    const cxchar *filename3 = "test_plist.fits";
    /*const cxchar regex[] = "^(DATE-OBS|ESO|COMMENT|HISTORY)$";*/
    const char to_rm[] = "^HIERARCH ESO |^NBXIS1$|^NBXIS2$|^HISTORY$";
    cpl_propertylist *plist, *_plist, *plist2;
    cpl_property *pro, *pro2;
    cpl_table *t;
    cpl_error_code code;

    fitsfile *fptr = NULL;
    fitsfile *_fptr;

    struct fcard {
        const cxchar *key;
        const cxchar *val;
        const cxchar *com;
        cpl_type type;
    };

    /*    struct fcard hdr[] = {
        {"SIMPLE", "T",
         "Standard FITS format (NOST-100.0)",
         CPL_TYPE_BOOL},
        {"BITPIX", "16",
         "# of bits storing pix values",
         CPL_TYPE_INT},
        {"NAXIS", "2",
         "# of axes in frame",
         CPL_TYPE_INT},
        {"NAXIS1", "2148",
         "# pixels/axis",
         CPL_TYPE_INT},
        {"NAXIS2", "2340",
         "# pixels/axis",
         CPL_TYPE_INT},
        {"ORIGIN", "ESO",
         "European Southern Observatory",
         CPL_TYPE_STRING},
        {"DATE", "2002-03-08T04:27:21.420",
         "Date this file was written (dd/mm/yyyy)",
         CPL_TYPE_STRING},
        {"MJD-OBS", "52341.17813019",
         "Obs start 2002-03-08T04:16:30.448",
         CPL_TYPE_DOUBLE},
        {"DATE-OBS", "2002-03-08T04:16:30.448",
         "Date of observation",
         CPL_TYPE_STRING},
        {"EXPTIME", "600.000",
         "Total integration time. 00:10:00.000",
         CPL_TYPE_DOUBLE},
        {"TELESCOP", "VLT",
         "ESO <TEL>",
         CPL_TYPE_STRING},
        {"RA", "181.41734",
         "12:05:40.1 RA (J2000) pointing",
         CPL_TYPE_DOUBLE},
        {"DEC", "-7.65555",
         "-07:39:19.9  DEC (J2000) pointing",
         CPL_TYPE_DOUBLE},
        {"EQUINOX", "2000.",
         "Standard FK5 (years)",
         CPL_TYPE_DOUBLE},
        {"RADECSYS", "FK5",
         "Coordinate reference frame",
         CPL_TYPE_STRING},
        {"LST", "38309.370",
         "10:38:29.370 LST at start",
         CPL_TYPE_DOUBLE},
        {"UTC", "15438.000",
         "04:17:18.000 UT at start",
         CPL_TYPE_DOUBLE},
        {"OBSERVER", "UNKNOWN",
         "Name of observer",
         CPL_TYPE_STRING},
        {"INSTRUME", "UNKNOWN",
         "Instrument used",
         CPL_TYPE_STRING},
        {"PI-COI", "'555555555'",
         "Name of PI and COI",
         CPL_TYPE_STRING},
        {"OBJECT", "None",
         "Original target",
         CPL_TYPE_STRING},
        {"PCOUNT", "0",
         "Number of parameters per group",
         CPL_TYPE_INT},
        {"GCOUNT", "1",
         "Number of groups",
         CPL_TYPE_INT},
        {"CRVAL1", "181.41734",
         "12:05:40.1, RA at ref pixel",
         CPL_TYPE_DOUBLE},
        {"CRPIX1", "2341.8585366",
         "Reference pixel in X",
         CPL_TYPE_DOUBLE},
        {"CDELT1", "0.20500000",
         "SS arcsec per pixel in RA",
         CPL_TYPE_DOUBLE},
        {"CTYPE1", "RA---TAN",
         "pixel coordinate system",
         CPL_TYPE_STRING},
        {"CRVAL2", "-7.65555",
         "-07:39:19.9, DEC at ref pixel",
         CPL_TYPE_DOUBLE},
        {"CRPIX2", "2487.8585366",
         "Reference pixel in Y",
         CPL_TYPE_DOUBLE},
        {"CDELT2", "0.20500000",
         "SS arcsec per pixel in DEC",
         CPL_TYPE_DOUBLE},
        {"CTYPE2", "DEC--TAN",
         "pixel coordinate system",
         CPL_TYPE_STRING},
        {"BSCALE", "1.0",
         "pixel=FITS*BSCALE+BZERO",
         CPL_TYPE_DOUBLE},
        {"BZERO", "32768.0",
         "pixel=FITS*BSCALE+BZERO",
         CPL_TYPE_DOUBLE},
        {"CD1_1", "0.000057",
         "Translation matrix element",
         CPL_TYPE_DOUBLE},
        {"CD1_2", "0.000000",
         "Translation matrix element",
         CPL_TYPE_DOUBLE},
        {"CD2_1", "0.000000",
         "Translation matrix element",
         CPL_TYPE_DOUBLE},
        {"CD2_2", "0.000057",
         "Translation matrix element",
         CPL_TYPE_DOUBLE},
        {"HIERARCH ESO OBS DID", "ESO-VLT-DIC.OBS-1.7",
         "OBS Dictionary",
         CPL_TYPE_STRING},
        {"HIERARCH ESO OBS OBSERVER", "UNKNOWN",
         "Observer Name",
         CPL_TYPE_STRING},
        {"HIERARCH ESO OBS PI-COI NAME", "UNKNOWN",
         "PI-COI name",
         CPL_TYPE_STRING},
        {"HIERARCH ESO INS GRAT NAME", "HR",
         "Grating name",
         CPL_TYPE_STRING},
        {"HIERARCH ESO PRO CATG", "X",
         "Product category",
         CPL_TYPE_STRING},
        {"HIERARCH ESO TPL NEXP", "5",
         "Number of exposures",
         CPL_TYPE_INT},
        {"HISTORY", "1st history record", NULL, CPL_TYPE_STRING},
        {"COMMENT", "1st comment record", NULL, CPL_TYPE_STRING},
        {"HISTORY", "2st history record", NULL, CPL_TYPE_STRING},
        {"COMMENT", "2st comment record", NULL, CPL_TYPE_STRING},
        {"COMMENT", "3st comment record", NULL, CPL_TYPE_STRING},
        {"HISTORY", "3st history record", NULL, CPL_TYPE_STRING},
        {"END", NULL, NULL, CPL_TYPE_STRING}
    };*/

    /*FILE *file;*/

    const cxchar *longname = "0123456789012345678901234567890123456789"
        "0123456789012345678901234567890123456789";

    cpl_init(CPL_INIT_DEFAULT);


    /*
     * Test 1: Create a property list and check its validity.
     */

    plist = cpl_propertylist_new();

    cx_assert(plist != NULL);
    cx_assert(cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 0);


    /*
     * Test 2: Append properties to the list created in the previous test
     *         and verify the data.
     */

    cpl_propertylist_append_char(plist, keys[0], 'a');
    cpl_propertylist_set_comment(plist, keys[0], comments[0]);

    cpl_propertylist_append_bool(plist, keys[1], 1);
    cpl_propertylist_set_comment(plist, keys[1], comments[1]);

    cpl_propertylist_append_int(plist, keys[2], -1);
    cpl_propertylist_set_comment(plist, keys[2], comments[2]);

    cpl_propertylist_append_long(plist, keys[3], 32768);
    cpl_propertylist_set_comment(plist, keys[3], comments[3]);

    cpl_propertylist_append_float(plist, keys[4], fval[0]);
    cpl_propertylist_set_comment(plist, keys[4], comments[4]);

    cpl_propertylist_append_double(plist, keys[5], dval[0]);
    cpl_propertylist_set_comment(plist, keys[5], comments[5]);

    cpl_propertylist_append_string(plist, keys[6], comments[6]);
    cpl_propertylist_set_comment(plist, keys[6], comments[6]);

    cx_assert(!cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 7);


    for (i = 0; i < cpl_propertylist_get_size(plist); i++) {
        cpl_property *p = cpl_propertylist_get(plist, i);

        cx_assert(!strcmp(cpl_property_get_name(p), keys[i]));
        cx_assert(!strcmp(cpl_property_get_comment(p), comments[i]));
        cx_assert(cpl_property_get_type(p) == types[i]);

        cx_assert(cpl_propertylist_has(plist, keys[i]));
        cx_assert(!strcmp(cpl_propertylist_get_comment(plist, keys[i]),
                          comments[i]));
        cx_assert(cpl_propertylist_get_type(plist, keys[i]) == types[i]);
    }

    cx_assert(cpl_propertylist_get_char(plist, keys[0]) == 'a');
    cx_assert(cpl_propertylist_get_bool(plist, keys[1]) == 1);
    cx_assert(cpl_propertylist_get_int(plist, keys[2]) == -1);
    cx_assert(cpl_propertylist_get_long(plist, keys[3]) == 32768);

    fval[1] = cpl_propertylist_get_float(plist, keys[4]);
    cx_assert(!memcmp(&fval[0], &fval[1], sizeof(float)));

    dval[1] = cpl_propertylist_get_double(plist, keys[5]);
    cx_assert(!memcmp(&dval[0], &dval[1], sizeof(double)));

    cx_assert(!strcmp(cpl_propertylist_get_string(plist, keys[6]),
                      comments[6]));


    /*
     * Test 3: Modify the values of the property list entries
     *         and verify the data.
     */

    cx_assert(cpl_propertylist_set_char(plist, keys[0], 'b') == 0);
    cx_assert(cpl_propertylist_get_char(plist, keys[0]) == 'b');

    cx_assert(cpl_propertylist_set_bool(plist, keys[1], 0) == 0);
    cx_assert(cpl_propertylist_get_bool(plist, keys[1]) == 0);

    cx_assert(cpl_propertylist_set_int(plist, keys[2], -1) == 0);
    cx_assert(cpl_propertylist_get_int(plist, keys[2]) == -1);

    cx_assert(cpl_propertylist_set_long(plist, keys[3], 1) == 0);
    cx_assert(cpl_propertylist_get_long(plist, keys[3]) == 1);

    fval[0] = 9.87654321;
    cx_assert(cpl_propertylist_set_float(plist, keys[4], fval[0]) == 0);
    fval[1] = cpl_propertylist_get_float(plist, keys[4]);
    cx_assert(!memcmp(&fval[0], &fval[1], sizeof(float)));

    dval[0] = -9.87654321;
    cx_assert(cpl_propertylist_set_double(plist, keys[5], dval[0]) == 0);
    dval[1] = cpl_propertylist_get_double(plist, keys[5]);
    cx_assert(!memcmp(&dval[0], &dval[1], sizeof(double)));

    cx_assert(cpl_propertylist_set_string(plist, keys[6], comments[0]) == 0);
    cx_assert(!strcmp(cpl_propertylist_get_string(plist, keys[6]),
                      comments[0]));


    /*
     * Test 4: Check that trying to modify an entry with a different
     *         type is properly failing.
     */

    cx_assert(cpl_propertylist_set_char(plist, keys[1], 'a') ==
              CPL_ERROR_TYPE_MISMATCH);
    cx_assert(cpl_propertylist_set_bool(plist, keys[2], 1) ==
              CPL_ERROR_TYPE_MISMATCH);
    cx_assert(cpl_propertylist_set_int(plist, keys[3], 1) ==
              CPL_ERROR_TYPE_MISMATCH);
    cx_assert(cpl_propertylist_set_long(plist, keys[4], 1) ==
              CPL_ERROR_TYPE_MISMATCH);
    cx_assert(cpl_propertylist_set_float(plist, keys[5], 1.) ==
              CPL_ERROR_TYPE_MISMATCH);
    cx_assert(cpl_propertylist_set_double(plist, keys[6], 1.) ==
              CPL_ERROR_TYPE_MISMATCH);
    cx_assert(cpl_propertylist_set_string(plist, keys[0], comments[0]) ==
              CPL_ERROR_TYPE_MISMATCH);


    /*
     * Test 5: Verify that values are inserted correctly into the property
     *         list.
     */

    cx_assert(cpl_propertylist_insert_char(plist, keys[0],
                                           keys[7], 'a') == 0);
    cx_assert(cpl_propertylist_insert_after_char(plist, keys[0],
                                                 keys[7], 'c') == 0);

    cx_assert(cpl_propertylist_insert_bool(plist, keys[1],
                                           keys[8], 0) == 0);
    cx_assert(cpl_propertylist_insert_after_bool(plist, keys[1],
                                                 keys[8], 1) == 0);

    cx_assert(cpl_propertylist_insert_int(plist, keys[2],
                                          keys[9], 0) == 0);
    cx_assert(cpl_propertylist_insert_after_int(plist, keys[2],
                                                keys[9], 1) == 0);

    cx_assert(cpl_propertylist_insert_long(plist, keys[3], keys[10],
                                           123456789) == 0);
    cx_assert(cpl_propertylist_insert_after_long(plist, keys[3], keys[10],
                                                 123456789) == 0);

    cx_assert(cpl_propertylist_insert_float(plist, keys[4], keys[11],
                                            fval[0]) == 0);
    cx_assert(cpl_propertylist_insert_after_float(plist, keys[4], keys[11],
                                                  -fval[0]) == 0);

    cx_assert(cpl_propertylist_insert_double(plist, keys[5], keys[12],
                                             dval[0]) == 0);
    cx_assert(cpl_propertylist_insert_after_double(plist, keys[5], keys[12],
                                                   -dval[0]) == 0);

    cx_assert(cpl_propertylist_insert_string(plist, keys[6],
                                             keys[13], "") == 0);
    cx_assert(cpl_propertylist_insert_after_string(plist, keys[6],
                                                   keys[13], "") == 0);

    for (i = 0; i < 7; i++) {
        cpl_property *p0 = cpl_propertylist_get(plist, 3 * i);
        cpl_property *p1 = cpl_propertylist_get(plist, 3 * i + 1);
        cpl_property *p2 = cpl_propertylist_get(plist, 3 * i + 2);

        cx_assert(!strcmp(cpl_property_get_name(p0), keys[i + 7]));
        cx_assert(!strcmp(cpl_property_get_name(p1), keys[i]));
        cx_assert(!strcmp(cpl_property_get_name(p2), keys[i + 7]));

        switch (cpl_property_get_type(p0)) {
        case CPL_TYPE_CHAR:
            cx_assert(cpl_property_get_char(p0) == 'a');
            cx_assert(cpl_property_get_char(p2) == 'c');
            break;

        case CPL_TYPE_BOOL:
            cx_assert(cpl_property_get_bool(p0) == 0);
            cx_assert(cpl_property_get_bool(p2) == 1);
            break;

        case CPL_TYPE_INT:
            cx_assert(cpl_property_get_int(p0) == 0);
            cx_assert(cpl_property_get_int(p2) == 1);
            break;

        case CPL_TYPE_LONG:
            cx_assert(cpl_property_get_long(p0) == 123456789);
            cx_assert(cpl_property_get_long(p2) == 123456789);
            break;

        case CPL_TYPE_FLOAT:
            fval[1] = cpl_property_get_float(p0);
            cx_assert(!memcmp(&fval[0], &fval[1], sizeof(float)));

            fval[1] = -cpl_property_get_float(p2);
            cx_assert(!memcmp(&fval[0], &fval[1], sizeof(float)));
            break;

        case CPL_TYPE_DOUBLE:
            dval[1] = cpl_property_get_double(p0);
            cx_assert(!memcmp(&dval[0], &dval[1], sizeof(double)));

            dval[1] = -cpl_property_get_double(p2);
            cx_assert(!memcmp(&dval[0], &dval[1], sizeof(double)));
            break;

        case CPL_TYPE_STRING:
            cx_assert(!strcmp(cpl_property_get_string(p0), ""));
            cx_assert(!strcmp(cpl_property_get_string(p2), ""));
            break;

        default:
            /* This point should never be reached */
            cx_log("cpl_propertylist-test", CX_LOG_LEVEL_ERROR, "file %s: "
                   "line %d: Invalid value type encountered", __FILE__,
                   __LINE__);
            break;
        }
    }


    /*
     * Test 6: Verify that modification of or insertion at/after a non
     *         existing elements is reported correctly.
     */

    cx_assert(cpl_propertylist_has(plist, "Non-existing key") == 0);

    cx_assert(cpl_propertylist_set_char(plist, "Non-existing key", 'a') ==
              CPL_ERROR_DATA_NOT_FOUND);
    cx_assert(cpl_propertylist_set_bool(plist, "Non-existing key", 1) ==
              CPL_ERROR_DATA_NOT_FOUND);
    cx_assert(cpl_propertylist_set_int(plist, "Non-existing key", 1) ==
              CPL_ERROR_DATA_NOT_FOUND);
    cx_assert(cpl_propertylist_set_long(plist, "Non-existing key", 1) ==
              CPL_ERROR_DATA_NOT_FOUND);
    cx_assert(cpl_propertylist_set_float(plist, "Non-existing key", 1) ==
              CPL_ERROR_DATA_NOT_FOUND);
    cx_assert(cpl_propertylist_set_double(plist, "Non-existing key", 1) ==
              CPL_ERROR_DATA_NOT_FOUND);
    cx_assert(cpl_propertylist_set_string(plist, "Non-existing key", "") ==
              CPL_ERROR_DATA_NOT_FOUND);

    cx_assert(cpl_propertylist_insert_char(plist, "Non-existing key",
                                           "h", 'a') == 1);
    cx_assert(cpl_propertylist_insert_bool(plist, "Non-existing key",
                                           "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_int(plist, "Non-existing key",
                                          "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_long(plist, "Non-existing key",
                                           "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_float(plist, "Non-existing key",
                                            "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_double(plist, "Non-existing key",
                                             "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_string(plist, "Non-existing key",
                                             "h", "") == 1);

    cx_assert(cpl_propertylist_insert_after_char(plist, "Non-existing key",
                                                 "h", 'a') == 1);
    cx_assert(cpl_propertylist_insert_after_bool(plist, "Non-existing key",
                                                 "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_after_int(plist, "Non-existing key",
                                                "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_after_long(plist, "Non-existing key",
                                                 "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_after_float(plist, "Non-existing key",
                                                  "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_after_double(plist, "Non-existing key",
                                                   "h", 1) == 1);
    cx_assert(cpl_propertylist_insert_after_string(plist, "Non-existing key",
                                                   "h", "") == 1);


    /*
     * Test 7: Create a copy of the property list and verify that original
     *         and copy are identical but do not share any resources.
     */

    _plist = cpl_propertylist_duplicate(plist);
    plist2 = cpl_propertylist_duplicate(plist);
    cx_assert(_plist != NULL);
    cx_assert(_plist != plist);
    cx_assert(plist2 != NULL);
    cx_assert(plist2 != plist);

    for (i = 0; i < cpl_propertylist_get_size(plist); i++) {
        cpl_property *p = cpl_propertylist_get(plist, i);
        cpl_property *_p = cpl_propertylist_get(_plist, i);

        cx_assert(cpl_property_get_name(p) != cpl_property_get_name(_p));
        cx_assert(!strcmp(cpl_property_get_name(p),
                          cpl_property_get_name(_p)));
        cx_assert(cpl_property_get_comment(p) == NULL ||
                  (cpl_property_get_comment(p) !=
                   cpl_property_get_comment(_p)));
        cx_assert(cpl_property_get_comment(p) == NULL ||
                  !strcmp(cpl_property_get_comment(p),
                          cpl_property_get_comment(_p)));

        switch (cpl_property_get_type(p)) {
        case CPL_TYPE_CHAR:
            cx_assert(cpl_property_get_char(p) ==
                      cpl_property_get_char(_p));
            break;

        case CPL_TYPE_BOOL:
            cx_assert(cpl_property_get_bool(p) ==
                      cpl_property_get_bool(_p));
            break;

        case CPL_TYPE_INT:
            cx_assert(cpl_property_get_int(p) ==
                      cpl_property_get_int(_p));
            break;

        case CPL_TYPE_LONG:
            cx_assert(cpl_property_get_long(p) ==
                      cpl_property_get_long(_p));
            break;

        case CPL_TYPE_FLOAT:
            fval[0] = cpl_property_get_float(p);
            fval[1] = cpl_property_get_float(_p);
            cx_assert(!memcmp(&fval[0], &fval[1], sizeof(float)));
            break;

        case CPL_TYPE_DOUBLE:
            dval[0] = cpl_property_get_double(p);
            dval[1] = cpl_property_get_double(_p);
            cx_assert(!memcmp(&dval[0], &dval[1], sizeof(double)));
            break;

        case CPL_TYPE_STRING:
            cx_assert(!strcmp(cpl_property_get_string(p),
                              cpl_property_get_string(_p)));
            break;

        default:
            /* This point should never be reached */
            cx_log("cpl_propertylist-test", CX_LOG_LEVEL_ERROR, "file %s: "
                   "line %d: Invalid value type encountered", __FILE__,
                   __LINE__);
            break;
        }
    }

    cpl_propertylist_delete(_plist);


    /*
     * Test 8: Erase elements from the property list and verify the list
     *         structure and the data. Each key exists twice in capital
     *         letters and once in lowercase letters.
     */

    for (i = 0; i < 7; i++) {
        cpl_propertylist_erase(plist, keys[i + 7]);
        cx_assert(cpl_propertylist_has(plist, keys[i + 7]) == 1);

        cpl_propertylist_erase(plist, keys[i + 7]);
        cx_assert(cpl_propertylist_has(plist, keys[i + 7]) == 0);
    }
    cx_assert(cpl_propertylist_get_size(plist) == 7);

    for (i = 0; i < 7; i++) {
        cpl_property *p = cpl_propertylist_get(plist, i);
        cx_assert(!strcmp(cpl_property_get_name(p), keys[i]));
    }

    cx_assert(cpl_propertylist_get_char(plist, keys[0]) == 'b');
    cx_assert(cpl_propertylist_get_bool(plist, keys[1]) == 0);
    cx_assert(cpl_propertylist_get_int(plist, keys[2]) == -1);
    cx_assert(cpl_propertylist_get_long(plist, keys[3]) == 1);

    fval[0] = 9.87654321;
    fval[1] = cpl_propertylist_get_float(plist, keys[4]);
    cx_assert(!memcmp(&fval[0], &fval[1], sizeof(float)));

    dval[0] = -9.87654321;
    dval[1] = cpl_propertylist_get_double(plist, keys[5]);
    cx_assert(!memcmp(&dval[0], &dval[1], sizeof(double)));

    cx_assert(!strcmp(cpl_propertylist_get_string(plist, keys[6]),
                      comments[0]));

    /*
     * Test 9: Erase all elements from the property list and verify that
     *         the list is empty.
     */

    cpl_propertylist_empty(plist);

    cx_assert(cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 0);

    cpl_propertylist_delete(plist);

     /*
     * Test 10: Write a propertylist to a FITS file
     *          and save it in disk
     */

    cx_assert(cpl_propertylist_update_string(plist2,"ESO OBS DID", "BLABLA") == CPL_ERROR_NONE);
    cx_assert(cpl_propertylist_update_string(plist2,"ESO OBS OBSERVER", "SANDRA") == CPL_ERROR_NONE);
    cx_assert(cpl_propertylist_update_string(plist2,"ESO OBS PI-COI NAME", "CASTRO") == CPL_ERROR_NONE);
    cx_assert(cpl_propertylist_update_string(plist2,"ESO INS GRAT NAME", "DODO") == CPL_ERROR_NONE);
    cx_assert(cpl_propertylist_update_string(plist2,"ESO PRO CATG", "JITTER_PRODUCT") == CPL_ERROR_NONE);
    cx_assert(cpl_propertylist_update_int(plist2,"ESO TPL NEXP", 4) == CPL_ERROR_NONE);
    cx_assert(cpl_propertylist_update_int(plist2, "TOFITS", 6) == CPL_ERROR_NONE);
    cx_assert(cpl_propertylist_set_comment(plist2, "TOFITS", "Test TOFITS function") == CPL_ERROR_NONE);

    remove(filename3);
    status = 0;

    fits_create_diskfile(&fptr, filename3, &status);
    cx_assert(status == 0);

    /*
    * Create simple HDU
    */
    fits_create_img(fptr, 8, 0, naxes, &status);
    cx_assert(status == 0);

    /*Save a propertylist to a FITS file*/
    cx_assert(cpl_propertylist_to_fitsfile(fptr, plist2, NULL) == CPL_ERROR_NONE);

    fits_close_file(fptr, &status);
    cpl_propertylist_delete(plist2);

    /* Test 11: Load back the propertylist from a FITS file using CFITSIO
    *          and compare it with a plist loaded using CPL.
    *         Retrieve a property usign its name
    */

    plist2 = cpl_propertylist_load(filename3, 0);
    cx_assert(plist2 != NULL);
    
    /* Try to load a non-existent extension and check that the proper error code is set */
    plist = cpl_propertylist_load(filename3, 10);
    cx_assert(cpl_error_get_code() == CPL_ERROR_DATA_NOT_FOUND);
    cpl_propertylist_delete(plist);

    pro2 = cpl_propertylist_get_property(plist2, "TOFITS");
    cx_assert(pro2 != NULL);
    cx_assert(strcmp("TOFITS",cpl_property_get_name(pro2)) == 0);
    cx_assert(cpl_property_get_int(pro2) == 6);

    status = 0;
    fits_open_diskfile(&_fptr, filename3, READONLY, &status);
    cx_assert(status == 0);

    _plist = cpl_propertylist_from_fitsfile(_fptr);
    cx_assert(_plist != NULL);

    /* Check that the property of both lists are the same */
    pro = cpl_propertylist_get_property(_plist, "TOFITS");
    cx_assert(pro != NULL);

    cx_assert(cpl_property_get_int(pro) == cpl_property_get_int(pro2));
    //    cpl_propertylist_delete(plist2);

    fits_close_file(_fptr, &status);
    /*
    * Test 12: Compare the properties of both lists
    */

    cx_assert(cpl_propertylist_get_size(plist2) ==
              cpl_propertylist_get_size(_plist));

    for (i = 0; i < cpl_propertylist_get_size(plist2); i++) {
        cpl_property *p = cpl_propertylist_get(plist2, i);
        cpl_property *_p = cpl_propertylist_get(_plist, i);

        cx_assert(strcmp(cpl_property_get_name(p),
                         cpl_property_get_name(_p)) == 0);
        cx_assert(strcmp(cpl_property_get_comment(p),
                         cpl_property_get_comment(_p)) == 0);
        cx_assert(cpl_property_get_type(p) == cpl_property_get_type(_p));

        switch (cpl_property_get_type(p)) {
        case CPL_TYPE_BOOL:
            cx_assert(cpl_property_get_bool(p) == cpl_property_get_bool(_p));
            break;

        case CPL_TYPE_INT:
            cx_assert(cpl_property_get_int(p) == cpl_property_get_int(_p));
            break;

        case CPL_TYPE_DOUBLE:
            cx_assert(cpl_property_get_double(p) ==
                      cpl_property_get_double(_p));
            break;

        case CPL_TYPE_STRING:
            cx_assert(strcmp(cpl_property_get_string(p),
                             cpl_property_get_string(_p)) == 0);
            break;

        default:
            cx_log("cpl_propertylist-test", CX_LOG_LEVEL_ERROR,
                   "file %s: line %d: Invalid value type encountered",
                   __FILE__, __LINE__);
            break;
        }
    }

    cpl_propertylist_delete(_plist);
    plist = cpl_propertylist_duplicate(plist2);

    cpl_propertylist_delete(plist2);

    /*
     * Test 13: Copy all propertys matching a given pattern from one
     *          property list to another.
     */

    _plist = cpl_propertylist_new();

    cpl_propertylist_copy_property_regexp(_plist, plist, "^ESO .*", 0);
    /* cx_assert(cpl_propertylist_get_size(_plist) == cpl_propertylist_get_size(plist));*/
    cx_assert(cpl_propertylist_has(_plist, "ESO OBS DID") != 0);
    cx_assert(cpl_propertylist_has(_plist, "ESO OBS OBSERVER") != 0);
    cx_assert(cpl_propertylist_has(_plist, "ESO OBS PI-COI NAME") != 0);
    cx_assert(cpl_propertylist_has(_plist, "ESO INS GRAT NAME") != 0);
    cx_assert(cpl_propertylist_has(_plist, "ESO PRO CATG") != 0);
    cx_assert(cpl_propertylist_has(_plist, "ESO TPL NEXP") != 0);

    cpl_propertylist_empty(_plist);
    cx_assert(cpl_propertylist_is_empty(_plist) != 0);

    cpl_propertylist_copy_property_regexp(_plist, plist, "^ESO .*", 1);
    cx_assert(cpl_propertylist_has(_plist, "ESO OBS DID") == 0);


    /*
     * Test 14a: Erase all properties matching the given pattern from the
     *          property list.
     */

    cpl_propertylist_empty(_plist);
    cx_assert(cpl_propertylist_is_empty(_plist) != 0);

    cpl_propertylist_copy_property_regexp(_plist, plist, "^ESO .*", 0);
    cx_assert(cpl_propertylist_get_size(_plist) == 6);

    cpl_propertylist_erase_regexp(_plist, "^ESO OBS .*", 0);
    cx_assert(cpl_propertylist_get_size(_plist) == 3);

    cpl_propertylist_erase_regexp(_plist, "ESO TPL NEXP", 0);
    cx_assert(cpl_propertylist_get_size(_plist) == 2);

    /*
    *  Test 14b: Erase a non-existing property and check return.
                 When regexp is null, the return should be -1
    */
    status = 0;
    status = cpl_propertylist_erase_regexp(_plist, "ESO TPL NEXP", 0);
    cx_assert(status == 0);
#if CPL_MAJOR_VERSION >= 5    
    cx_assert(cpl_propertylist_erase_regexp(_plist, NULL, 0) == -1);
#else
    cx_assert(cpl_propertylist_erase_regexp(_plist, NULL, 0) == 0);
#endif    
    cpl_propertylist_delete(_plist);
    cpl_propertylist_delete(plist);


    /*
     * Test 15: Create a property list from a file. Only properties matching
     *          the given pattern are loaded.
     */

    plist = NULL;

    plist = cpl_propertylist_load_regexp(filename3, 0,
                                         "^ESO .*", 0);
    cx_assert(plist != NULL);
    cx_assert(cpl_propertylist_is_empty(plist) == 0);
    cx_assert(cpl_propertylist_get_size(plist) == 6);
    cx_assert(cpl_propertylist_has(plist, "ESO OBS DID") != 0);
    cx_assert(cpl_propertylist_has(plist, "ESO OBS OBSERVER") != 0);
    cx_assert(cpl_propertylist_has(plist, "ESO OBS PI-COI NAME") != 0);
    cx_assert(cpl_propertylist_has(plist, "ESO INS GRAT NAME") != 0);
    cx_assert(cpl_propertylist_has(plist, "ESO PRO CATG") != 0);
    cx_assert(cpl_propertylist_has(plist, "ESO TPL NEXP") != 0);

    cpl_propertylist_delete(plist);


    /*
     * Test 16: Append a property list to another.
     */

    plist = cpl_propertylist_new();
    _plist = cpl_propertylist_new();

    cpl_propertylist_append_char(plist, keys[0], 'a');
    cpl_propertylist_set_comment(plist, keys[0], comments[0]);

    cpl_propertylist_append_bool(plist, keys[1], 1);
    cpl_propertylist_set_comment(plist, keys[1], comments[1]);

    cpl_propertylist_append_int(plist, keys[2], -1);
    cpl_propertylist_set_comment(plist, keys[2], comments[2]);

    cpl_propertylist_append_long(plist, keys[3], 32768);
    cpl_propertylist_set_comment(plist, keys[3], comments[3]);

    cpl_propertylist_append_float(_plist, keys[4], fval[0]);
    cpl_propertylist_set_comment(_plist, keys[4], comments[4]);

    cpl_propertylist_append_double(_plist, keys[5], dval[0]);
    cpl_propertylist_set_comment(_plist, keys[5], comments[5]);

    cpl_propertylist_append_string(_plist, keys[6], comments[6]);
    cpl_propertylist_set_comment(_plist, keys[6], comments[6]);

    cx_assert(!cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 4);

    cx_assert(!cpl_propertylist_is_empty(_plist));
    cx_assert(cpl_propertylist_get_size(_plist) == 3);

    cpl_propertylist_append(plist, _plist);

    cx_assert(!cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 7);

    cx_assert(!cpl_propertylist_is_empty(_plist));
    cx_assert(cpl_propertylist_get_size(_plist) == 3);

    for (i = 0; i < cpl_propertylist_get_size(plist); i++) {
        cpl_property *p = cpl_propertylist_get(plist, i);

        cx_assert(!strcmp(cpl_property_get_name(p), keys[i]));
        cx_assert(!strcmp(cpl_property_get_comment(p), comments[i]));
        cx_assert(cpl_property_get_type(p) == types[i]);

        cx_assert(cpl_propertylist_has(plist, keys[i]));
        cx_assert(!strcmp(cpl_propertylist_get_comment(plist, keys[i]),
                          comments[i]));
        cx_assert(cpl_propertylist_get_type(plist, keys[i]) == types[i]);
    }

    /*
    * Test 17: Sequentially access the elements of a property list
    */

    /* First element */

    /*    pro = cpl_propertylist_get_first(plist);
    cx_assert(pro != NULL);

    cx_assert(!strcmp(cpl_property_get_name(pro), keys[0]));
    cx_assert(!strcmp(cpl_property_get_comment(pro), comments[0]));
    cx_assert(cpl_property_get_type(pro) == types[0]);*/

    /*
    * Test 18: Loop through all elements except the first,
    *           and check the results.
    */
    /*    for (i = 1; i < cpl_propertylist_get_size(plist); i++) {

        cpl_property *p = cpl_propertylist_get_next(plist);

        cx_assert(!strcmp(cpl_property_get_name(p), keys[i]));
        cx_assert(!strcmp(cpl_property_get_comment(p), comments[i]));
        cx_assert(cpl_property_get_type(p) == types[i]);

        cx_assert(cpl_propertylist_has(plist, keys[i]));
        cx_assert(!strcmp(cpl_propertylist_get_comment(plist, keys[i]),
                          comments[i]));
        cx_assert(cpl_propertylist_get_type(plist, keys[i]) == types[i]);
     }*/

     /*
    * Test 19: Try to access the next element and see
    *           if the end of list is handled properly.
    */

    /*    pro = cpl_propertylist_get_next(plist);
    cx_assert(pro == NULL);*/


/*
     * Test 20: Create a FITS header using a list containing a property with
     *          a name of length 80 characters (the length of a FITS card)
     */

    cpl_propertylist_empty(plist);

    cx_assert(cpl_propertylist_append_string(plist, longname, comments[6]) == CPL_ERROR_NONE);

    cpl_propertylist_delete(_plist);
    _plist = NULL;

    cpl_propertylist_delete(plist);
    plist = NULL;

    /*
    * Test 21:  Sort a property list
    */

    plist = cpl_propertylist_new();

    cx_assert(plist != NULL);
    cx_assert(cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 0);
    

    for(i = 0; i < 1; i++) {

        cpl_propertylist_append_int(plist, "FDEBACGH", -1);
        cpl_propertylist_set_comment(plist, keys[2], comments[2]);

        cpl_propertylist_append_char(plist, "CDEFGB1", 'a');
        cpl_propertylist_set_comment(plist, keys[0], comments[0]);

        cpl_propertylist_append_bool(plist, "ABCDEFGH", 1);
        cpl_propertylist_set_comment(plist, keys[1], comments[1]);
        
        cpl_propertylist_append_float(plist, "ZZZGDBCA", fval[0]);
        cpl_propertylist_set_comment(plist, keys[4], comments[4]);

        cpl_propertylist_append_string(plist, "BBBBB2", comments[6]);
        cpl_propertylist_set_comment(plist, keys[6], comments[6]);

        cpl_propertylist_append_long(plist, "HISTORY", 32768);
        cpl_propertylist_set_comment(plist, keys[3], comments[3]);
        
        cpl_propertylist_append_int(plist, "HISTORY", 0);
        cpl_propertylist_set_comment(plist, keys[3], comments[3]);

        cpl_propertylist_append_double(plist, "HISTORY", dval[0]);
        cpl_propertylist_set_comment(plist, keys[5], comments[5]);

    }

    cx_assert(!cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 8);

    cx_assert(cpl_propertylist_sort(plist, cpl_test_property_comparison) == CPL_ERROR_NONE);

    cpl_propertylist_delete(plist);
   
    /*
    * Test 21a:  Save a propertylist which contains a HISTORY
    *            field without a value
    */
    
    plist = cpl_propertylist_new();
    cpl_propertylist_append_string(plist, "HISTORY", "HELLO");
    cpl_propertylist_append_string(plist, "HISTORY", "WORLD");
    cpl_propertylist_append_string(plist, "HISTORY", "");
    cpl_propertylist_append_string(plist, "HISTORY", "HELLO");
    cpl_propertylist_append_string(plist, "HISTORY", "AGAIN");
    
    cpl_propertylist_append_string(plist, "COMMENT", "HELLO");
    cpl_propertylist_append_string(plist, "COMMENT", "WORLD");
    cpl_propertylist_append_string(plist, "COMMENT", "");
    cpl_propertylist_append_string(plist, "COMMENT", "HELLO");
    cpl_propertylist_append_string(plist, "COMMENT", "AGAIN");
    

    remove("test_plist21a.fits");
    t = cpl_table_new(1);
    cpl_table_save(t, plist, NULL,"test_plist21a.fits", CPL_IO_DEFAULT);
    
    
    cpl_propertylist_delete(plist);
    cpl_table_delete(t); 
 
    /*
    * Test 21b: load plist from an image and sort it.
    * This test was compared with result from cx_list_sort.
    * Results are the same.
    */
    
    /*    plist = cpl_propertylist_load("image.fits", 0);
    cx_assert(plist !=NULL);
    
    cx_assert(cpl_propertylist_sort(plist, cpl_test_property_comparison) == CPL_ERROR_NONE);

    
    cpl_image_save(NULL, "sorted.fits", CPL_BPP_8_UNSIGNED, plist, CPL_IO_DEFAULT);

 
    cpl_propertylist_delete(plist);*/
    
    
    /*
    * Test 21c:  Save a very long property list into a FITS file
    */

    /*    plist = cpl_propertylist_new();

    cx_assert(plist != NULL);
    cx_assert(cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 0);

    for(i = 0; i < 8000; i++) {
        cpl_propertylist_append_float(plist, keys[4], fval[0]);
        cpl_propertylist_set_comment(plist, keys[4], comments[4]);

        cpl_propertylist_append_int(plist, keys[2], -1);
        cpl_propertylist_set_comment(plist, keys[2], comments[2]);

        cpl_propertylist_append_char(plist, keys[0], 'a');
        cpl_propertylist_set_comment(plist, keys[0], comments[0]);

        cpl_propertylist_append_bool(plist, keys[1], 1);
        cpl_propertylist_set_comment(plist, keys[1], comments[1]);

        cpl_propertylist_append_string(plist, keys[6], comments[6]);
        cpl_propertylist_set_comment(plist, keys[6], comments[6]);

        cpl_propertylist_append_long(plist, keys[3], 32768);
        cpl_propertylist_set_comment(plist, keys[3], comments[3]);

        cpl_propertylist_append_double(plist, keys[5], dval[0]);
        cpl_propertylist_set_comment(plist, keys[5], comments[5]);

    }

    cx_assert(!cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 56000);

    remove(filename3);
    status = 0;

    fits_create_diskfile(&fptr, filename3, &status);
    cx_assert(status == 0);


    fits_create_img(fptr, 8, 0, naxes, &status);
    cx_assert(status == 0);
    system("timeInSecs");

    cx_assert(cpl_propertylist_to_fitsfile(fptr, plist, NULL) == CPL_ERROR_NONE);
    system("timeInSecs");

    fits_close_file(fptr, &status);
    cpl_propertylist_delete(plist);*/

    /*
    *   Test 22: Save a property list into a FITS file using
    *            cpl_propertylist_save()
    */
 
    plist = cpl_propertylist_new();
    cpl_propertylist_append_char(plist, keys[0], 'a');
    cpl_propertylist_set_comment(plist, keys[0], comments[0]);

    cpl_propertylist_append_bool(plist, keys[1], 1);
    cpl_propertylist_set_comment(plist, keys[1], comments[1]);

    cpl_propertylist_append_int(plist, keys[2], -1);
    cpl_propertylist_set_comment(plist, keys[2], comments[2]);

    cpl_propertylist_append_long(plist, keys[3], 32768);
    cpl_propertylist_set_comment(plist, keys[3], comments[3]);

    cpl_propertylist_append_float(plist, keys[4], fval[0]);
    cpl_propertylist_set_comment(plist, keys[4], comments[4]);

    cpl_propertylist_append_double(plist, keys[5], dval[0]);
    cpl_propertylist_set_comment(plist, keys[5], comments[5]);

    cpl_propertylist_append_string(plist, keys[6], comments[6]);
    cpl_propertylist_set_comment(plist, keys[6], comments[6]);

    cx_assert(!cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 7);


    code = cpl_propertylist_save(plist, "test_plist22.fits", CPL_IO_DEFAULT);
    cx_assert(code == CPL_ERROR_NONE);
    
    
    cpl_propertylist_update_string(plist, keys[6], "updated string");
    code = cpl_propertylist_save(plist, "test_plist22.fits", CPL_IO_EXTEND);
    cx_assert(code == CPL_ERROR_NONE);

    cpl_propertylist_delete(plist);
      
    /*
    *   Test 23: Save a property list to a FITS file using
    *            a regexp to remove keys
    */
    remove(filename3);
    status = 0;

    fits_create_diskfile(&fptr, filename3, &status);
    cx_assert(status == 0);

    /*
    * Create simple HDU
    */
    fits_create_img(fptr, 8, 0, naxes, &status);
    cx_assert(status == 0);

    plist = cpl_propertylist_new();
    cpl_propertylist_append_char(plist, keys[0], 'a');
    cpl_propertylist_set_comment(plist, keys[0], comments[0]);

    cpl_propertylist_append_bool(plist, keys[1], 1);
    cpl_propertylist_set_comment(plist, keys[1], comments[1]);

    cpl_propertylist_append_int(plist, keys[2], -1);
    cpl_propertylist_set_comment(plist, keys[2], comments[2]);

    cpl_propertylist_append_long(plist, "NBXIS1", 32768);
    cpl_propertylist_set_comment(plist, "NBXIS1", comments[3]);

    cpl_propertylist_append_float(plist, keys[4], fval[0]);
    cpl_propertylist_set_comment(plist, keys[4], comments[4]);
    cpl_propertylist_append_string(plist, "HIERARCH ESO TEST1", "One string without any comment");
    cpl_propertylist_append_string(plist, "HIERARCH ESO TEST2", "Two string without any comment");

    cpl_propertylist_append_double(plist, keys[5], dval[0]);
    cpl_propertylist_set_comment(plist, keys[5], comments[5]);

    cpl_propertylist_append_long(plist, "NBXIS2", 32769);
    cpl_propertylist_set_comment(plist, "NBXIS2", comments[3]);

    cpl_propertylist_append_string(plist, "HISTORY", "One history string without any comment");
    cpl_propertylist_append_string(plist, "HISTORY", "Two history string without any comment");
    cpl_propertylist_append_string(plist, "HISTORY", "Three history string without any comment");
    
    cx_assert(!cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 12);
    
    /* Remove the following keys from the saving
       HIERARCH ESO |NBXIS1|NBXIS2|HISTORY
    */    
    
    cx_assert(cpl_propertylist_to_fitsfile(fptr, plist, to_rm) == CPL_ERROR_NONE);
    
    /* Pass a null regexp to the function*/
    plist2 = cpl_propertylist_duplicate(plist);
    cx_assert(cpl_propertylist_to_fitsfile(fptr, plist2, NULL) == CPL_ERROR_NONE);
    
    /*Pass an empty property list to the function */
    cpl_propertylist_delete(plist2);
    plist2 = cpl_propertylist_new();
    cpl_propertylist_to_fitsfile(fptr, plist2, NULL);
    
    fits_close_file(fptr, &status);
    cpl_propertylist_delete(plist);
    cpl_propertylist_delete(plist2);

    /*
    *   Test 24: Create a property list with compressed
    *            keywords and save it in disk and in a table.
    *            Check if compressed
    *            keywords are removed after saving it.
    */

    remove("test_plist24.fits");
    plist = cpl_propertylist_new();
    cpl_propertylist_append_string(plist,"ZTENSION", "COMPRESSED_EMPTY");
    cpl_propertylist_append_long(plist, "ZNAXIS", 2);
    cpl_propertylist_set_comment(plist, "ZNAXIS", "compressed NAXIS");
    cpl_propertylist_append_long(plist, "ZNAXIS1", 2048);
    cpl_propertylist_set_comment(plist, "ZNAXIS1", "compressed NAXIS1");
    cpl_propertylist_append_long(plist, "ZNAXIS2", 1024);
    cpl_propertylist_set_comment(plist, "ZNAXIS2", "compressed NAXIS2");
    cx_assert(!cpl_propertylist_is_empty(plist));
    cx_assert(cpl_propertylist_get_size(plist) == 4);

    
    /* Save it in disk and check if compression keywords are removed */
    cx_assert(cpl_propertylist_save(plist, "test_plist24.fits", CPL_IO_DEFAULT) == CPL_ERROR_NONE);
    cx_assert(cpl_propertylist_save(plist, "test_plist24.fits", CPL_IO_EXTEND) == CPL_ERROR_NONE);
    cpl_propertylist_delete(plist);

    /* primary header */
    plist = cpl_propertylist_load("test_plist24.fits", 0);
    cx_assert(cpl_propertylist_has(plist, "ZTENSION") == 0);
    cpl_propertylist_delete(plist);
    
    /*extension header */
    plist = cpl_propertylist_load("test_plist24.fits", 1);
    cx_assert(cpl_propertylist_has(plist, "ZNAXIS") == 0);

    
    remove("test_plist24.fits");
    t = cpl_table_new(1);

    /* Compressed keywords should be removed when saving table*/
    cx_assert(cpl_table_save(t,plist,NULL,"test_plist24.fits",CPL_IO_DEFAULT) == CPL_ERROR_NONE);
    cx_assert(cpl_table_save(t,NULL,plist,"test_plist24.fits",CPL_IO_EXTEND) == CPL_ERROR_NONE);
    cpl_propertylist_delete(plist);

    /* Check if compressed keywords were removed from headers*/
    plist = cpl_propertylist_load("test_plist24.fits",1);
    cx_assert(cpl_propertylist_has(plist, "ZTENSION") == 0);
    cx_assert(cpl_propertylist_has(plist, "ZNAXIS") == 0);
    cx_assert(cpl_propertylist_has(plist, "ZNAXIS1") == 0);
    cx_assert(cpl_propertylist_has(plist, "ZNAXIS2") == 0);
    cpl_propertylist_delete(plist);
    
    plist = cpl_propertylist_load("test_plist24.fits",2);
    cx_assert(cpl_propertylist_has(plist, "ZTENSION") == 0);
    cx_assert(cpl_propertylist_has(plist, "ZNAXIS") == 0);
    cx_assert(cpl_propertylist_has(plist, "ZNAXIS1") == 0);
    cx_assert(cpl_propertylist_has(plist, "ZNAXIS2") == 0);

    cpl_table_delete(t);
    cpl_propertylist_delete(plist);
    
    
    /*
     * All tests succeeded
     */
    
    cpl_end();
    return 0;

}
