/* $Id: cpl_apertures-test.c,v 1.20 2007/12/11 11:58:39 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2004 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2007/12/11 11:58:39 $
 * $Revision: 1.20 $
 * $Name:  $
 */

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <math.h>

#include <cpl_image_gen.h>
#include <cpl_image_io.h>
#include <cpl_image_bpm.h>
#include <cpl_stats.h>
#include <cpl_msg.h>
#include <cpl_memory.h>
#include <cpl_mask.h>
#include <cpl_tools.h>

#include "cpl_apertures.h"

/*-----------------------------------------------------------------------------
                                   Define
 -----------------------------------------------------------------------------*/

#define IMAGESZ     10
#define IMAGESZ2    512

/*-----------------------------------------------------------------------------
                                  Main
 -----------------------------------------------------------------------------*/
int main(void)
{
    cpl_image       *   imd ;
    cpl_image       *   labels ;
    cpl_mask        *   bin ;
    cpl_apertures   *   aperts ;
    double          *   val ;
    const double        psigmas[] = {5, 2, 1, 0.5};
    cpl_vector      *   sigmas;
    int                 i, j, pos ;

    cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);

    /* Insert tests below */


    /* Create test image and the labels */
    cpl_msg_info("","Create a %dx%d test image", IMAGESZ, IMAGESZ) ;
    val = cpl_malloc(IMAGESZ*IMAGESZ*sizeof(double)) ;
    for (i=0 ; i<IMAGESZ ; i++) {
        for (j=0 ; j<IMAGESZ ; j++) {
            pos = i + j * IMAGESZ ;
            val[pos] = 0 ;
        }
    }
    val[33] = val[34] = val[35] = val[36] = 1.0 ;
    val[43] = val[44] = val[45] = val[46] = 1.0 ;
    val[53] = val[54] = val[56] = 1.0 ;
    val[55] = 10.0 ;
    val[63] = val[64] = val[65] = val[66] = 1.0 ;
    val[88] = 20.0 ;
    imd = cpl_image_wrap_double(IMAGESZ, IMAGESZ, val) ;

    /* Test with a 'bad' label image */
    labels = cpl_image_new(cpl_image_get_size_x(imd),
                           cpl_image_get_size_y(imd), CPL_TYPE_INT) ;
    cpl_test_nonnull(labels);

    aperts = cpl_apertures_new_from_image(imd, labels);
    cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
    cpl_test_null(aperts);

    cpl_image_set(labels, 1, 1, 1.0);
    cpl_test_error(CPL_ERROR_NONE);

    aperts = cpl_apertures_new_from_image(imd, labels);
    cpl_test_error(CPL_ERROR_NONE);
    cpl_test_nonnull(aperts);

    cpl_apertures_delete(aperts) ;
    cpl_image_set(labels, 1, 1, 2.0) ;
    cpl_test_error(CPL_ERROR_NONE);

    aperts = cpl_apertures_new_from_image(imd, labels);
    cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
    cpl_test_null(aperts);

    cpl_image_delete(labels) ;

    /* Thresholding */
    bin = cpl_mask_threshold_image_create(imd, 0.5, DBL_MAX) ;
    cpl_test_nonnull(bin);
    labels = cpl_image_labelise_mask_create(bin, NULL) ;
    cpl_test_nonnull(labels);
    cpl_mask_delete(bin) ;

    /* Compute statistics on the apertures */
    cpl_msg_info("","Compute statistics on detected apertures") ;
    aperts = cpl_apertures_new_from_image(imd, labels);
    cpl_test_error(CPL_ERROR_NONE);
    cpl_test_nonnull(aperts);

    /* cpl_apertures_dump(aperts, stdout) ; */
    cpl_apertures_delete(aperts) ;
    
    /* Set a bad pixel and recompute the apertures */
    cpl_image_reject(imd, 6, 6) ;
    cpl_test_error(CPL_ERROR_NONE);

    aperts = cpl_apertures_new_from_image(imd, labels);
    cpl_test_error(CPL_ERROR_NONE);
    cpl_test_nonnull(aperts);

    /* cpl_apertures_dump(aperts, stdout) ; */
    cpl_image_delete(labels) ;

    /* Get the biggest object */
    cpl_msg_info("","Get the biggest aperture:") ;
    cpl_test_eq(cpl_apertures_sort_by_npix(aperts), CPL_ERROR_NONE);
    cpl_test_error(CPL_ERROR_NONE);
    cpl_msg_info("","number of pixels: %d", cpl_apertures_get_npix(aperts, 1)) ;

    /* Get the brightest object */
    cpl_msg_info("","Get the brightest aperture:") ;
    cpl_test_eq(cpl_apertures_sort_by_flux(aperts), CPL_ERROR_NONE);
    cpl_test_error(CPL_ERROR_NONE);
    cpl_msg_info("","flux: %g", cpl_apertures_get_flux(aperts, 1)) ;

    /* Get the aperture with the highest peak */
    cpl_msg_info("","Get the aperture with the highest peak:") ;
    cpl_test_eq(cpl_apertures_sort_by_max(aperts), CPL_ERROR_NONE);
    cpl_test_error(CPL_ERROR_NONE);
    cpl_msg_info("","maximum value: %g", cpl_apertures_get_max(aperts, 1)) ;
    cpl_image_delete(imd) ;
    cpl_apertures_delete(aperts) ;
 
    /* Create a new test image */
    imd = cpl_image_fill_test_create(IMAGESZ2, IMAGESZ2) ;
    
    sigmas = cpl_vector_wrap(4, (double*)psigmas);

    /* Test cpl_apertures_extract() */
    cpl_msg_info("","Test the apertures detection with a thresholding method") ;
    aperts = cpl_apertures_extract(imd, sigmas, &pos);
    cpl_test_error(CPL_ERROR_NONE);
    cpl_test_nonnull(aperts);

    /* cpl_apertures_dump(aperts, stdout) ; */
    cpl_apertures_delete(aperts) ;

    /* Test cpl_apertures_extract_window() */
    cpl_msg_info("","Test the apertures detection (threshold) on a zone") ;
    aperts = cpl_apertures_extract_window(imd, sigmas, IMAGESZ2/4, IMAGESZ2/4,
                                          3*IMAGESZ2/4, 3*IMAGESZ2/4, &pos);
    cpl_test_error(CPL_ERROR_NONE);
    cpl_test_nonnull(aperts);

    /* cpl_apertures_dump(aperts, stdout) ; */
    cpl_vector_unwrap(sigmas);
    cpl_apertures_delete(aperts) ;
    cpl_image_delete(imd) ;

   /* End of tests */
    return cpl_test_end(0);
}
